(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@rx-angular/cdk/template'), require('@angular/core'), require('@rx-angular/cdk/coercing'), require('@rx-angular/cdk/notifications'), require('rxjs'), require('rxjs/operators'), require('@rx-angular/cdk/render-strategies')) :
    typeof define === 'function' && define.amd ? define('@rx-angular/template/if', ['exports', '@rx-angular/cdk/template', '@angular/core', '@rx-angular/cdk/coercing', '@rx-angular/cdk/notifications', 'rxjs', 'rxjs/operators', '@rx-angular/cdk/render-strategies'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global['rx-angular'] = global['rx-angular'] || {}, global['rx-angular'].template = global['rx-angular'].template || {}, global['rx-angular'].template.if = {}), global['rx-angular'].cdk.template, global.ng.core, global['rx-angular'].cdk.coercing, global['rx-angular'].cdk.notifications, global.rxjs, global.rxjs.operators, global['rx-angular'].cdk['render-strategies']));
}(this, (function (exports, template, i0, coercing, notifications, rxjs, operators, i1) { 'use strict';

    function _interopNamespace(e) {
        if (e && e.__esModule) return e;
        var n = Object.create(null);
        if (e) {
            Object.keys(e).forEach(function (k) {
                if (k !== 'default') {
                    var d = Object.getOwnPropertyDescriptor(e, k);
                    Object.defineProperty(n, k, d.get ? d : {
                        enumerable: true,
                        get: function () {
                            return e[k];
                        }
                    });
                }
            });
        }
        n['default'] = e;
        return Object.freeze(n);
    }

    var i0__namespace = /*#__PURE__*/_interopNamespace(i0);
    var i1__namespace = /*#__PURE__*/_interopNamespace(i1);

    var RxIfTemplateNames = Object.assign(Object.assign({}, template.RxBaseTemplateNames), { then: 'rxThen', else: 'rxElse' });

    /**
     * @Directive IfDirective
     * @description
     *
     * The `RxIf` directive is drop-in replacement for the `NgIf` directive, but with additional features.
     * `RxIf` allows you to bind observables directly without having the need of using the `async`
     * pipe in addition.
     *
     * This enables `rxIf` to completely operate on its own without having to interact with `NgZone`
     * or triggering global change detection.
     *
     * Read more about the RxIf directive in the [official
     *   docs](https://www.rx-angular.io/docs/template/api/rx-if-directive).
     *
     * @example
     * <app-item *rxIf="show$"></app-item>
     *
     * @docsCategory RxIf
     * @docsPage RxIf
     * @publicApi
     */
    // eslint-disable-next-line @angular-eslint/directive-class-suffix
    var RxIf = /** @class */ (function () {
        function RxIf(strategyProvider, cdRef, ngZone, templateRef, viewContainerRef) {
            this.strategyProvider = strategyProvider;
            this.cdRef = cdRef;
            this.ngZone = ngZone;
            this.templateRef = templateRef;
            this.viewContainerRef = viewContainerRef;
            /** @internal */
            this.subscription = new rxjs.Subscription();
            /**
             * @description
             *
             * Structural directives maintain `EmbeddedView`s within a components' template.
             * Depending on the bound value as well as the configured `RxRenderStrategy`,
             * updates processed by the `*rxIf` directive might be asynchronous.
             *
             * Whenever a template gets inserted into, or removed from, its parent component, the directive has to inform the
             *   parent in order to update any view- or contentquery (`@ViewChild`, `@ViewChildren`, `@ContentChild`,
             *   `@ContentChildren`).
             *
             * Read more about this in the
             * [official
             *   docs](https://www.rx-angular.io/docs/template/api/rx-if-directive#local-strategies-and-view-content-queries-parent).
             *
             * @example
             * \@Component({
             *   selector: 'app-root',
             *   template: `
             *    <app-component>
             *      <app-item
             *        *rxIf="
             *          show$;
             *          parent: true;
             *        "
             *      >
             *      </app-item>
             *    </app-component>
             *   `
             * })
             * export class AppComponent {
             *   show$ = state.select('showItem');
             * }
             *
             * @param {boolean} renderParent
             */
            this.renderParent = this.strategyProvider.config.parent;
            /**
             * @description
             * A flag to control whether `*rxIf` templates are created within `NgZone` or not.
             * The default value is `true, `*rxIf` will create its `EmbeddedView` inside `NgZone`.
             *
             * Event listeners normally trigger zone.
             * Especially high frequency events can cause performance issues.
             *
             * Read more about this in the
             * [official docs](https://www.rx-angular.io/docs/template/api/let-directive#working-with-event-listeners-patchzone).
             *
             * @example
             * \@Component({
             *   selector: 'app-root',
             *   template: `
             *    <app-component>
             *      <app-item
             *        *rxIf="
             *          show$;
             *          patchZone: false;
             *        "
             *        (drag)="itemDrag($event)"
             *      >
             *      </app-item>
             *    </app-component>
             *   `
             * })
             * export class AppComponent {
             *   show$ = state.select('showItem');
             * }
             *
             * @param {boolean} patchZone
             */
            this.patchZone = this.strategyProvider.config.patchZone;
            /** @internal */
            this.triggerHandler = new rxjs.ReplaySubject(1);
            /** @internal */
            this.templateNotifier = notifications.createTemplateNotifier();
            /** @internal */
            this.strategyHandler = coercing.coerceAllFactory(function () { return new rxjs.ReplaySubject(1); }, operators.mergeAll());
            /** @internal */
            this.rendered$ = new rxjs.Subject();
        }
        Object.defineProperty(RxIf.prototype, "strategy", {
            /**
             * @description
             *
             * You can change the used `RenderStrategy` by using the `strategy` input of the `*rxIf`. It accepts
             * an `Observable<RxStrategyNames>` or
             *   [`RxStrategyNames`](https://github.com/rx-angular/rx-angular/blob/b0630f69017cc1871d093e976006066d5f2005b9/libs/cdk/render-strategies/src/lib/model.ts#L52).
             *
             * The default value for strategy is
             * [`normal`](https://www.rx-angular.io/docs/template/cdk/render-strategies/strategies/concurrent-strategies).
             *
             * Read more about this in the
             * [official docs](https://www.rx-angular.io/docs/template/api/rx-if-directive#use-render-strategies-strategy).
             *
             * @example
             *
             * \@Component({
             *   selector: 'app-root',
             *   template: `
             *     <ng-container *rxIf="showHero$; strategy: 'userBlocking'">
             *       <app-hero></app-hero>
             *     </ng-container>
             *
             *     <ng-container *rxIf="showHero$; strategy: strategy$">
             *       <app-hero></app-hero>
             *     </ng-container>
             *   `
             * })
             * export class AppComponent {
             *   strategy$ = of('immediate');
             * }
             *
             * @param { string | Observable<string> | undefined } strategyName
             * @see {@link RxStrategyNames}
             */
            set: function (strategyName) {
                this.strategyHandler.next(strategyName);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(RxIf.prototype, "renderCallback", {
            /**
             * @description
             * A `Subject` which emits whenever `*rxIf` rendered a change to the view.
             * This enables developers to perform actions when rendering has been done.
             * The `renderCallback` is useful in situations where you
             * rely on specific DOM properties like the dimensions of an item after it got rendered.
             *
             * The `renderCallback` emits the latest value causing the view to update.
             *
             * @example
             * \@Component({
             *   selector: 'app-root',
             *   template: `
             *    <app-component>
             *      <app-item
             *        *rxIf="
             *          show$;
             *          renderCallback: rendered;
             *        "
             *      >
             *      </app-item>
             *    </app-component>
             *   `
             * })
             * export class AppComponent {
             *  show$ = state.select('showItem');
             *  // this emits whenever rxIf finished rendering changes
             *  rendered = new Subject<boolean>();
             *
             *   constructor(elementRef: ElementRef<HTMLElement>) {
             *     rendered.subscribe(() => {
             *       // item is rendered, we can access its dom now
             *     })
             *   }
             * }
             *
             * @param {Subject<boolean>} callback
             */
            set: function (callback) {
                this._renderObserver = callback;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(RxIf.prototype, "thenTemplate", {
            /** @internal */
            get: function () {
                return this.then ? this.then : this.templateRef;
            },
            enumerable: false,
            configurable: true
        });
        /** @internal */
        RxIf.prototype.ngOnInit = function () {
            var _this = this;
            var _a, _b, _c, _d;
            this.subscription.add(rxjs.merge(this.contextTrigger || rxjs.NEVER, ((_a = this.nextTrigger) === null || _a === void 0 ? void 0 : _a.pipe(operators.map(function () { return "next"; } /* Next */))) || rxjs.NEVER, ((_b = this.suspenseTrigger) === null || _b === void 0 ? void 0 : _b.pipe(operators.map(function () { return "suspense"; } /* Suspense */))) ||
                rxjs.NEVER, ((_c = this.completeTrigger) === null || _c === void 0 ? void 0 : _c.pipe(operators.map(function () { return "complete"; } /* Complete */))) ||
                rxjs.NEVER, ((_d = this.errorTrigger) === null || _d === void 0 ? void 0 : _d.pipe(operators.map(function () { return "error"; } /* Error */))) || rxjs.NEVER)
                .pipe(operators.filter(function (v) { return !!v; }))
                .subscribe(function (t) { return _this.triggerHandler.next(t); }));
            this.subscription.add(this.templateManager
                .render(this.templateNotifier.values$)
                .subscribe(function (n) {
                var _a;
                _this.rendered$.next(n);
                (_a = _this._renderObserver) === null || _a === void 0 ? void 0 : _a.next(n);
            }));
        };
        /** @internal */
        RxIf.prototype.ngOnChanges = function (changes) {
            if (!this.templateManager) {
                this._createTemplateManager();
            }
            if (changes.then && !changes.then.firstChange) {
                this.templateManager.addTemplateRef(RxIfTemplateNames.then, this.thenTemplate);
            }
            if (changes.else) {
                this.templateManager.addTemplateRef(RxIfTemplateNames.else, this.else);
            }
            if (changes.complete) {
                this.templateManager.addTemplateRef(RxIfTemplateNames.complete, this.complete);
            }
            if (changes.suspense) {
                this.templateManager.addTemplateRef(RxIfTemplateNames.suspense, this.suspense);
                this.templateNotifier.withInitialSuspense(!!this.suspense);
            }
            if (changes.error) {
                this.templateManager.addTemplateRef(RxIfTemplateNames.error, this.error);
            }
            if (changes.rxIf) {
                this.templateNotifier.next(this.rxIf);
            }
        };
        /** @internal */
        RxIf.prototype.ngOnDestroy = function () {
            this.subscription.unsubscribe();
        };
        /** @internal */
        RxIf.prototype._createTemplateManager = function () {
            var _e;
            var _this = this;
            var getNextTemplate = function (value) {
                return value
                    ? RxIfTemplateNames.then
                    : _this.else
                        ? RxIfTemplateNames.else
                        : undefined;
            };
            this.templateManager = template.createTemplateManager({
                templateSettings: {
                    viewContainerRef: this.viewContainerRef,
                    customContext: function (rxIf) { return ({ rxIf: rxIf }); },
                },
                renderSettings: {
                    cdRef: this.cdRef,
                    parent: coerceBooleanProperty(this.renderParent),
                    patchZone: this.patchZone ? this.ngZone : false,
                    defaultStrategyName: this.strategyProvider.primaryStrategy,
                    strategies: this.strategyProvider.strategies,
                },
                notificationToTemplateName: (_e = {},
                    _e["suspense" /* Suspense */] = function (value) { return _this.suspense ? RxIfTemplateNames.suspense : getNextTemplate(value); },
                    _e["next" /* Next */] = function (value) { return getNextTemplate(value); },
                    _e["error" /* Error */] = function (value) { return _this.error ? RxIfTemplateNames.error : getNextTemplate(value); },
                    _e["complete" /* Complete */] = function (value) { return _this.complete ? RxIfTemplateNames.complete : getNextTemplate(value); },
                    _e),
                templateTrigger$: this.triggerHandler,
            });
            this.templateManager.addTemplateRef(RxIfTemplateNames.then, this.thenTemplate);
            this.templateManager.nextStrategy(this.strategyHandler.values$);
        };
        /**
         * Asserts the correct type of the context for the template that `NgIf` will render.
         *
         * The presence of this method is a signal to the Ivy template type-check compiler that the
         * `NgIf` structural directive renders its template with a specific context type.
         */
        // eslint-disable-next-line @typescript-eslint/member-ordering
        RxIf.ngTemplateContextGuard = function (dir, ctx) {
            return true;
        };
        return RxIf;
    }());
    /** @nocollapse */ RxIf.ɵfac = i0__namespace.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0__namespace, type: RxIf, deps: [{ token: i1__namespace.RxStrategyProvider }, { token: i0__namespace.ChangeDetectorRef }, { token: i0__namespace.NgZone }, { token: i0__namespace.TemplateRef }, { token: i0__namespace.ViewContainerRef }], target: i0__namespace.ɵɵFactoryTarget.Directive });
    /** @nocollapse */ RxIf.ɵdir = i0__namespace.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.16", type: RxIf, selector: "[rxIf]", inputs: { rxIf: "rxIf", strategy: ["rxIfStrategy", "strategy"], else: ["rxIfElse", "else"], then: ["rxIfThen", "then"], suspense: ["rxIfSuspense", "suspense"], complete: ["rxIfComplete", "complete"], error: ["rxIfError", "error"], contextTrigger: ["rxIfContextTrigger", "contextTrigger"], nextTrigger: ["rxIfNextTrigger", "nextTrigger"], suspenseTrigger: ["rxIfSuspenseTrigger", "suspenseTrigger"], errorTrigger: ["rxIfErrorTrigger", "errorTrigger"], completeTrigger: ["rxIfCompleteTrigger", "completeTrigger"], renderParent: ["rxIfParent", "renderParent"], patchZone: ["rxIfPatchZone", "patchZone"], renderCallback: ["rxIfRenderCallback", "renderCallback"] }, usesOnChanges: true, ngImport: i0__namespace });
    i0__namespace.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0__namespace, type: RxIf, decorators: [{
                type: i0.Directive,
                args: [{
                        selector: '[rxIf]',
                    }]
            }], ctorParameters: function () { return [{ type: i1__namespace.RxStrategyProvider }, { type: i0__namespace.ChangeDetectorRef }, { type: i0__namespace.NgZone }, { type: i0__namespace.TemplateRef }, { type: i0__namespace.ViewContainerRef }]; }, propDecorators: { rxIf: [{
                    type: i0.Input
                }], strategy: [{
                    type: i0.Input,
                    args: ['rxIfStrategy']
                }], else: [{
                    type: i0.Input,
                    args: ['rxIfElse']
                }], then: [{
                    type: i0.Input,
                    args: ['rxIfThen']
                }], suspense: [{
                    type: i0.Input,
                    args: ['rxIfSuspense']
                }], complete: [{
                    type: i0.Input,
                    args: ['rxIfComplete']
                }], error: [{
                    type: i0.Input,
                    args: ['rxIfError']
                }], contextTrigger: [{
                    type: i0.Input,
                    args: ['rxIfContextTrigger']
                }], nextTrigger: [{
                    type: i0.Input,
                    args: ['rxIfNextTrigger']
                }], suspenseTrigger: [{
                    type: i0.Input,
                    args: ['rxIfSuspenseTrigger']
                }], errorTrigger: [{
                    type: i0.Input,
                    args: ['rxIfErrorTrigger']
                }], completeTrigger: [{
                    type: i0.Input,
                    args: ['rxIfCompleteTrigger']
                }], renderParent: [{
                    type: i0.Input,
                    args: ['rxIfParent']
                }], patchZone: [{
                    type: i0.Input,
                    args: ['rxIfPatchZone']
                }], renderCallback: [{
                    type: i0.Input,
                    args: ['rxIfRenderCallback']
                }] } });
    /**
     * @internal
     * @description
     * Coerces a data-bound value (typically a string) to a boolean.
     *
     */
    function coerceBooleanProperty(value) {
        return value != null && "" + value !== 'false';
    }

    var DECLARATIONS = [RxIf];
    var IfModule = /** @class */ (function () {
        function IfModule() {
        }
        return IfModule;
    }());
    /** @nocollapse */ IfModule.ɵfac = i0__namespace.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0__namespace, type: IfModule, deps: [], target: i0__namespace.ɵɵFactoryTarget.NgModule });
    /** @nocollapse */ IfModule.ɵmod = i0__namespace.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0__namespace, type: IfModule, declarations: [RxIf], exports: [RxIf] });
    /** @nocollapse */ IfModule.ɵinj = i0__namespace.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0__namespace, type: IfModule, imports: [[]] });
    i0__namespace.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0__namespace, type: IfModule, decorators: [{
                type: i0.NgModule,
                args: [{
                        declarations: DECLARATIONS,
                        imports: [],
                        exports: DECLARATIONS,
                    }]
            }] });

    /**
     * Generated bundle index. Do not edit.
     */

    exports.IfModule = IfModule;
    exports.RxIf = RxIf;
    exports.RxIfTemplateNames = RxIfTemplateNames;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=template-if.umd.js.map
