(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@rx-angular/cdk/coercing'), require('@rx-angular/cdk/notifications'), require('@rx-angular/cdk/template'), require('rxjs'), require('rxjs/operators'), require('@rx-angular/cdk/render-strategies')) :
    typeof define === 'function' && define.amd ? define('@rx-angular/template/let', ['exports', '@angular/core', '@rx-angular/cdk/coercing', '@rx-angular/cdk/notifications', '@rx-angular/cdk/template', 'rxjs', 'rxjs/operators', '@rx-angular/cdk/render-strategies'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global['rx-angular'] = global['rx-angular'] || {}, global['rx-angular'].template = global['rx-angular'].template || {}, global['rx-angular'].template.let = {}), global.ng.core, global['rx-angular'].cdk.coercing, global['rx-angular'].cdk.notifications, global['rx-angular'].cdk.template, global.rxjs, global.rxjs.operators, global['rx-angular'].cdk['render-strategies']));
}(this, (function (exports, i0, coercing, notifications, template, rxjs, operators, i1) { 'use strict';

    function _interopNamespace(e) {
        if (e && e.__esModule) return e;
        var n = Object.create(null);
        if (e) {
            Object.keys(e).forEach(function (k) {
                if (k !== 'default') {
                    var d = Object.getOwnPropertyDescriptor(e, k);
                    Object.defineProperty(n, k, d.get ? d : {
                        enumerable: true,
                        get: function () {
                            return e[k];
                        }
                    });
                }
            });
        }
        n['default'] = e;
        return Object.freeze(n);
    }

    var i0__namespace = /*#__PURE__*/_interopNamespace(i0);
    var i1__namespace = /*#__PURE__*/_interopNamespace(i1);

    /** @internal */
    var RxLetTemplateNames = Object.assign(Object.assign({}, template.RxBaseTemplateNames), { next: 'nextTpl' });
    /**
     * @Directive LetDirective
     *
     * @description
     * In Angular there is one way to handle asynchronous values or streams in the template, the `async` pipe.
     * Even though the async pipe evaluates such values in the template, it is insufficient in many ways.
     * To name a few:
     * * it will only update the template when `NgZone` is also aware of the value change
     * * it leads to over rendering because it can only run global change detection
     * * it leads to too many subscriptions in the template
     * * it is cumbersome to work with values in the template
     *
     * read more about the LetDirective in the [official docs](https://www.rx-angular.io/docs/template/api/let-directive)
     *
     * **Conclusion - Structural directives**
     *
     * In contrast to global change detection, structural directives allow fine-grained control of change detection on a per directive basis.
     * The `LetDirective` comes with its own way to handle change detection in templates in a very efficient way.
     * However, the change detection behavior is configurable on a per directive or global basis.
     * This makes it possible to implement your own strategies, and also provides a migration path from large existing apps running with Angulars default change detection.
     *
     * This package helps to reduce code used to create composable action streams.
     * It mostly is used in combination with state management libs to handle user interaction and backend communication.
     *
     * ```html
     * <ng-container *rxLet="observableNumber$; let n">
     *  ...
     * </ng-container>
     * ```
     *
     *
     * @docsCategory LetDirective
     * @docsPage LetDirective
     * @publicApi
     */
    var LetDirective = /** @class */ (function () {
        function LetDirective(strategyProvider, cdRef, ngZone, nextTemplateRef, viewContainerRef, errorHandler) {
            var _this = this;
            this.strategyProvider = strategyProvider;
            this.cdRef = cdRef;
            this.ngZone = ngZone;
            this.nextTemplateRef = nextTemplateRef;
            this.viewContainerRef = viewContainerRef;
            this.errorHandler = errorHandler;
            /**
             * @description
             *
             * When local rendering strategies are used, we need to treat view and content queries in a
             * special way.
             * To make `*rxLet` in such situations, a certain mechanism is implemented to
             * execute change detection on the parent (`parent`).
             *
             * This is required if your components state is dependent on its view or content children:
             *
             * - `@ViewChild`
             * - `@ViewChildren`
             * - `@ContentChild`
             * - `@ContentChildren`
             *
             * Read more about this in the
             * [official
             * docs](https://www.rx-angular.io/docs/template/api/let-directive#local-strategies-and-view-content-queries-parent).
             *
             * @example
             * \@Component({
             *   selector: 'app-root',
             *   template: `
             *    <app-list-component>
             *      <app-list-item
             *        *rxLet="
             *          item$;
             *          let item;
             *          parent: true;
             *        "
             *      >
             *        <div>{{ item.name }}</div>
             *      </app-list-item>
             *    </app-list-component>
             *   `
             * })
             * export class AppComponent {
             *   item$ = itemService.getItem();
             * }
             *
             * @param boolean
             */
            this.renderParent = this.strategyProvider.config.parent;
            /**
             * @description
             * A flag to control whether *rxLet templates are created within `NgZone` or not.
             * The default value is `true, `*rxLet` will create it's `EmbeddedViews` inside `NgZone`.
             *
             * Event listeners normally trigger zone. Especially high frequently events cause performance issues.
             *
             * Read more about this in the
             * [official docs](https://www.rx-angular.io/docs/template/api/let-directive#working-with-event-listeners-patchzone).
             *
             * @example
             * \@Component({
             *   selector: 'app-root',
             *   template: `
             *    <app-list-component>
             *      <app-list-item
             *        *rxLet="
             *          item$;
             *          let item;
             *          patchZone: false;
             *        "
             *      >
             *        <div>{{ item.name }}</div>
             *      </app-list-item>
             *    </app-list-component>
             *   `
             * })
             * export class AppComponent {
             *   item$ = itemService.getItem();
             * }
             */
            this.patchZone = this.strategyProvider.config.patchZone;
            /** @internal */
            this.observablesHandler = notifications.createTemplateNotifier();
            /** @internal */
            this.strategyHandler = coercing.coerceAllFactory(function () { return new rxjs.ReplaySubject(1); });
            /** @internal */
            this.triggerHandler = new rxjs.ReplaySubject(1);
            /** @internal */
            this.subscription = new rxjs.Subscription();
            /** @internal */
            this.rendered$ = new rxjs.Subject();
            /** @internal */
            this.templateNotification$ = new rxjs.Subject();
            /** @internal */
            this.values$ = this.observablesHandler.values$;
            this.rendered = rxjs.defer(function () { return _this.rendered$; });
        }
        Object.defineProperty(LetDirective.prototype, "strategy", {
            /**
             * @description
             *
             * You can change the used `RenderStrategy` by using the `strategy` input of the `*rxLet`. It accepts
             * an `Observable<RxStrategyNames>` or [`RxStrategyNames`](https://github.com/rx-angular/rx-angular/blob/b0630f69017cc1871d093e976006066d5f2005b9/libs/cdk/render-strategies/src/lib/model.ts#L52).
             *
             * The default value for strategy is
             * [`normal`](https://www.rx-angular.io/docs/template/cdk/render-strategies/strategies/concurrent-strategies).
             *
             * Read more about this in the
             * [official docs](https://www.rx-angular.io/docs/template/api/let-directive#use-render-strategies-strategy).
             *
             * @example
             *
             * \@Component({
             *   selector: 'app-root',
             *   template: `
             *     <ng-container *rxLet="hero$; let hero; strategy: strategy">
             *       <app-hero [hero]="hero"></app-hero>
             *     </ng-container>
             *
             *     <ng-container *rxLet="hero$; let hero; strategy: strategy$">
             *       <app-hero [hero]="hero"></app-hero>
             *     </ng-container>
             *   `
             * })
             * export class AppComponent {
             *   strategy = 'low';
             *   strategy$ = of('immediate');
             * }
             *
             * @param { string | Observable<string> | undefined } strategyName
             * @see {@link RxStrategyNames}
             */
            set: function (strategyName) {
                this.strategyHandler.next(strategyName);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(LetDirective.prototype, "renderCallback", {
            /**
             * @description
             * A `Subject` which emits whenever *rxFor finished rendering a set changes to the view.
             * This enables developers to perform actions when a list has finished rendering.
             * The `renderCallback` is useful in situations where you rely on specific DOM properties like the `height` a
             * table after all items got rendered.
             * It is also possible to use the renderCallback in order to determine if a view should be visible or not. This
             * way developers can hide a list as long as it has not finished rendering.
             *
             * The result of the `renderCallback` will contain the currently rendered set of items in the iterable.
             *
             * @example
             * \Component({
             *   selector: 'app-root',
             *   template: `
             *   <app-list-component>
             *     <app-list-item
             *       *rxFor="
             *         let item of items$;
             *         trackBy: trackItem;
             *         renderCallback: itemsRendered;
             *       ">
             *       <div>{{ item.name }}</div>
             *     </app-list-item>
             *   </app-list-component>
             * `
             * })
             * export class AppComponent {
             *   items$: Observable<Item[]> = itemService.getItems();
             *   trackItem = (idx, item) => item.id;
             *   // this emits whenever rxFor finished rendering changes
             *   itemsRendered = new Subject<Item[]>();
             *
             *   constructor(elementRef: ElementRef<HTMLElement>) {
             *     itemsRendered.subscribe(() => {
             *       // items are rendered, we can now scroll
             *       elementRef.scrollTo({bottom: 0});
             *     })
             *   }
             * }
             *
             * @param {Subject<U>} renderCallback
             */
            set: function (callback) {
                this._renderObserver = callback;
            },
            enumerable: false,
            configurable: true
        });
        /** @internal */
        LetDirective.ngTemplateContextGuard = function (dir, ctx) {
            return true;
        };
        /** @internal */
        LetDirective.prototype.ngOnInit = function () {
            var _this = this;
            var _a, _b, _c, _d;
            this.subscription.add(this.templateManager
                .render(rxjs.merge(this.values$, this.templateNotification$))
                .subscribe(function (n) {
                var _a;
                _this.rendered$.next(n);
                (_a = _this._renderObserver) === null || _a === void 0 ? void 0 : _a.next(n);
            }));
            this.subscription.add(rxjs.merge(this.contextTrigger || rxjs.NEVER, ((_a = this.nextTrigger) === null || _a === void 0 ? void 0 : _a.pipe(operators.map(function () { return "next"; } /* Next */))) || rxjs.NEVER, ((_b = this.suspenseTrigger) === null || _b === void 0 ? void 0 : _b.pipe(operators.map(function () { return "suspense"; } /* Suspense */))) ||
                rxjs.NEVER, ((_c = this.completeTrigger) === null || _c === void 0 ? void 0 : _c.pipe(operators.map(function () { return "complete"; } /* Complete */))) ||
                rxjs.NEVER, ((_d = this.errorTrigger) === null || _d === void 0 ? void 0 : _d.pipe(operators.map(function () { return "error"; } /* Error */))) || rxjs.NEVER)
                .pipe(operators.filter(function (v) { return !!v; }))
                .subscribe(function (t) { return _this.triggerHandler.next(t); }));
        };
        /** @internal */
        LetDirective.prototype.ngOnChanges = function (changes) {
            if (!this.templateManager) {
                this._createTemplateManager();
            }
            if (changes.complete) {
                this.templateManager.addTemplateRef(RxLetTemplateNames.complete, this.complete);
            }
            if (changes.suspense) {
                this.templateManager.addTemplateRef(RxLetTemplateNames.suspense, this.suspense);
                this.observablesHandler.withInitialSuspense(!!this.suspense);
            }
            if (changes.error) {
                this.templateManager.addTemplateRef(RxLetTemplateNames.error, this.error);
            }
            if (changes.rxLet) {
                this.observablesHandler.next(this.rxLet);
            }
        };
        /** @internal */
        LetDirective.prototype.ngOnDestroy = function () {
            this.subscription.unsubscribe();
        };
        /** @internal */
        LetDirective.prototype._createTemplateManager = function () {
            var _e;
            var _this = this;
            this.templateManager = template.createTemplateManager({
                templateSettings: {
                    viewContainerRef: this.viewContainerRef,
                    customContext: function (rxLet) { return ({ rxLet: rxLet }); },
                },
                renderSettings: {
                    cdRef: this.cdRef,
                    parent: !!this.renderParent,
                    patchZone: this.patchZone ? this.ngZone : false,
                    defaultStrategyName: this.strategyProvider.primaryStrategy,
                    strategies: this.strategyProvider.strategies,
                    errorHandler: this.errorHandler,
                },
                notificationToTemplateName: (_e = {},
                    _e["suspense" /* Suspense */] = function () { return _this.suspense ? RxLetTemplateNames.suspense : RxLetTemplateNames.next; },
                    _e["next" /* Next */] = function () { return RxLetTemplateNames.next; },
                    _e["error" /* Error */] = function () { return _this.error ? RxLetTemplateNames.error : RxLetTemplateNames.next; },
                    _e["complete" /* Complete */] = function () { return _this.complete ? RxLetTemplateNames.complete : RxLetTemplateNames.next; },
                    _e),
                templateTrigger$: this.triggerHandler,
            });
            this.templateManager.addTemplateRef(RxLetTemplateNames.next, this.nextTemplateRef);
            this.templateManager.nextStrategy(this.strategyHandler.values$);
        };
        return LetDirective;
    }());
    /** @nocollapse */ LetDirective.ɵfac = i0__namespace.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0__namespace, type: LetDirective, deps: [{ token: i1__namespace.RxStrategyProvider }, { token: i0__namespace.ChangeDetectorRef }, { token: i0__namespace.NgZone }, { token: i0__namespace.TemplateRef }, { token: i0__namespace.ViewContainerRef }, { token: i0__namespace.ErrorHandler }], target: i0__namespace.ɵɵFactoryTarget.Directive });
    /** @nocollapse */ LetDirective.ɵdir = i0__namespace.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.16", type: LetDirective, selector: "[rxLet]", inputs: { rxLet: "rxLet", strategy: ["rxLetStrategy", "strategy"], complete: ["rxLetComplete", "complete"], error: ["rxLetError", "error"], suspense: ["rxLetSuspense", "suspense"], contextTrigger: ["rxLetContextTrigger", "contextTrigger"], completeTrigger: ["rxLetCompleteTrigger", "completeTrigger"], errorTrigger: ["rxLetErrorTrigger", "errorTrigger"], suspenseTrigger: ["rxLetSuspenseTrigger", "suspenseTrigger"], nextTrigger: ["rxLetNextTrigger", "nextTrigger"], renderCallback: ["rxLetRenderCallback", "renderCallback"], renderParent: ["rxLetParent", "renderParent"], patchZone: ["rxLetPatchZone", "patchZone"] }, outputs: { rendered: "rendered" }, usesOnChanges: true, ngImport: i0__namespace });
    i0__namespace.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0__namespace, type: LetDirective, decorators: [{
                type: i0.Directive,
                args: [{ selector: '[rxLet]' }]
            }], ctorParameters: function () { return [{ type: i1__namespace.RxStrategyProvider }, { type: i0__namespace.ChangeDetectorRef }, { type: i0__namespace.NgZone }, { type: i0__namespace.TemplateRef }, { type: i0__namespace.ViewContainerRef }, { type: i0__namespace.ErrorHandler }]; }, propDecorators: { rxLet: [{
                    type: i0.Input
                }], strategy: [{
                    type: i0.Input,
                    args: ['rxLetStrategy']
                }], complete: [{
                    type: i0.Input,
                    args: ['rxLetComplete']
                }], error: [{
                    type: i0.Input,
                    args: ['rxLetError']
                }], suspense: [{
                    type: i0.Input,
                    args: ['rxLetSuspense']
                }], contextTrigger: [{
                    type: i0.Input,
                    args: ['rxLetContextTrigger']
                }], completeTrigger: [{
                    type: i0.Input,
                    args: ['rxLetCompleteTrigger']
                }], errorTrigger: [{
                    type: i0.Input,
                    args: ['rxLetErrorTrigger']
                }], suspenseTrigger: [{
                    type: i0.Input,
                    args: ['rxLetSuspenseTrigger']
                }], nextTrigger: [{
                    type: i0.Input,
                    args: ['rxLetNextTrigger']
                }], renderCallback: [{
                    type: i0.Input,
                    args: ['rxLetRenderCallback']
                }], renderParent: [{
                    type: i0.Input,
                    args: ['rxLetParent']
                }], patchZone: [{
                    type: i0.Input,
                    args: ['rxLetPatchZone']
                }], rendered: [{
                    type: i0.Output
                }] } });

    var LetModule = /** @class */ (function () {
        function LetModule() {
        }
        return LetModule;
    }());
    /** @nocollapse */ LetModule.ɵfac = i0__namespace.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0__namespace, type: LetModule, deps: [], target: i0__namespace.ɵɵFactoryTarget.NgModule });
    /** @nocollapse */ LetModule.ɵmod = i0__namespace.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0__namespace, type: LetModule, declarations: [LetDirective], exports: [LetDirective] });
    /** @nocollapse */ LetModule.ɵinj = i0__namespace.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0__namespace, type: LetModule });
    i0__namespace.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0__namespace, type: LetModule, decorators: [{
                type: i0.NgModule,
                args: [{
                        declarations: [LetDirective],
                        exports: [LetDirective],
                    }]
            }] });

    /**
     * Generated bundle index. Do not edit.
     */

    exports.LetDirective = LetDirective;
    exports.LetModule = LetModule;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=template-let.umd.js.map
